"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.APIClient = void 0;
const sequelize_1 = require("sequelize");
const models_1 = require("../models");
const types_1 = require("../types");
const trojan_manager_1 = require("./trojan-manager");
class APIClient {
    trojanManager;
    constructor({ host, port }) {
        this.trojanManager = new trojan_manager_1.TrojanManager({ host, port });
    }
    async init({ onTickError, }) {
        await this.onTick();
        setInterval(() => {
            this.onTick().catch((err) => {
                if (err instanceof Error) {
                    try {
                        onTickError(err);
                    }
                    catch (_) {
                        // ignore
                    }
                }
            });
        }, 60 * 1000);
    }
    async listAccounts() {
        const accounts = await models_1.Account.findAll();
        return { type: types_1.ECommand.List, data: accounts };
    }
    async addAccount(accountId, passwordHash) {
        await models_1.Account.findOrCreate({
            where: { id: accountId },
            defaults: { id: accountId, password: passwordHash },
        });
        return { type: types_1.ECommand.Add, accountId };
    }
    async removeAccount(accountId) {
        await models_1.Account.destroy({
            where: {
                id: accountId,
            },
        });
        return { type: types_1.ECommand.Delete, accountId };
    }
    async changePassword(accountId, passwordHash) {
        await this.removeAccount(accountId);
        await this.addAccount(accountId, passwordHash);
        return {
            type: types_1.ECommand.ChangePassword,
            accountId,
            password: passwordHash,
        };
    }
    async getFlow(options = {}) {
        const startTime = options.startTime || 0;
        const endTime = options.endTime || Date.now();
        const accountFlows = await models_1.Account.findAll({
            include: [
                {
                    association: models_1.Account.associations.flows,
                    where: {
                        createdAt: {
                            [sequelize_1.Op.between]: [startTime, endTime],
                        },
                    },
                },
            ],
        });
        const results = accountFlows.map((account) => {
            const flows = account.flows;
            let flow = 0;
            if (flows) {
                flows.forEach((f) => {
                    flow += f.flow;
                });
            }
            return {
                accountId: account.id,
                flow,
            };
        });
        if (options.clear) {
            await models_1.Flow.destroy({
                where: {
                    createdAt: {
                        [sequelize_1.Op.between]: [startTime, endTime],
                    },
                },
            });
        }
        return { type: types_1.ECommand.Flow, data: results };
    }
    disconnect() {
        this.trojanManager.disconnect();
    }
    async onTick() {
        const [dbAccounts, trojanInMemoryAccountFlows] = await Promise.all([
            models_1.Account.findAll(),
            this.trojanManager.getFlows(),
        ]);
        const passwordsToAdd = [];
        const flowsToAdd = [];
        const passwordsToRemove = trojanInMemoryAccountFlows.map((f) => f.passwordHash);
        for await (const account of dbAccounts) {
            const { password } = account;
            const inMemoryAccountIndex = trojanInMemoryAccountFlows.findIndex((f) => f.passwordHash === password);
            const flows = trojanInMemoryAccountFlows
                .filter((flow) => flow.passwordHash === account.password)
                .map((flow) => ({
                accountId: account.id,
                flow: flow.flow,
            }));
            flowsToAdd.push(...flows);
            if (inMemoryAccountIndex > -1) {
                // Account exists in memory and database, keep it in the memory
                passwordsToRemove.splice(inMemoryAccountIndex, 1);
            }
            else {
                // Account doesn't exist in memory, add it to memory
                passwordsToAdd.push(password);
            }
        }
        await models_1.Flow.bulkCreate(flowsToAdd);
        // Reset all flows
        await this.trojanManager.clearFlow(trojanInMemoryAccountFlows.map((flow) => flow.passwordHash));
        if (passwordsToAdd.length) {
            await this.trojanManager.addAccount(passwordsToAdd);
        }
        if (passwordsToRemove.length) {
            // Passwords left in the array are not in the database, remove them from memory
            await this.trojanManager.removeAccount(passwordsToRemove);
        }
    }
}
exports.APIClient = APIClient;
//# sourceMappingURL=api-client.js.map