"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("source-map-support/register");
const death_1 = __importDefault(require("death"));
const net_1 = require("net");
const config_1 = require("./config");
const db_1 = require("./db");
const api_client_1 = require("./api-client");
const logger_1 = require("./logger");
const sentry_1 = __importDefault(require("./sentry"));
const socket_1 = require("./socket");
const trojan_1 = require("./trojan");
const types_1 = require("./types");
const utils_1 = require("./utils");
const version_1 = require("./version");
require("./models");
let config;
let trojanClient;
let trojanProcess = null;
let fakeWebsiteProcess = null;
/**
 * https://shadowsocks.github.io/shadowsocks-manager/#/ssmgrapi
 */
const receiveCommand = async (data) => {
    const message = {
        command: types_1.ECommand.Version,
        port: 0,
        password: '',
        ...data,
    };
    logger_1.logger.info('Message received: %j', message);
    switch (message.command) {
        case types_1.ECommand.List:
            return trojanClient.listAccounts();
        case types_1.ECommand.Add:
            return trojanClient.addAccount(message.port, message.password);
        case types_1.ECommand.Delete:
            return trojanClient.removeAccount(message.port);
        case types_1.ECommand.Flow:
            return trojanClient.getFlow(message.options);
        case types_1.ECommand.ChangePassword:
            return trojanClient.changePassword(message.port, message.password);
        case types_1.ECommand.Version:
            return { type: types_1.ECommand.Version, version: version_1.version };
        default:
            return (0, utils_1.assertNever)(message.command);
    }
};
const parseResult = (result) => {
    switch (result.type) {
        case types_1.ECommand.List:
            return result.data.map((user) => ({
                port: user.accountId,
                password: user.password,
            }));
        case types_1.ECommand.Add:
            return { port: result.accountId };
        case types_1.ECommand.Delete:
            return { port: result.accountId };
        case types_1.ECommand.Flow:
            return result.data.map((user) => ({
                port: user.accountId,
                sumFlow: user.flow,
            }));
        case types_1.ECommand.ChangePassword:
            return { port: result.accountId, password: result.password };
        case types_1.ECommand.Version:
            return { version: result.version };
        default:
            return (0, utils_1.assertNever)(result);
    }
};
const checkData = async (receive) => {
    const buffer = receive.data;
    let length = 0;
    let data;
    let code;
    if (buffer.length < 2) {
        return;
    }
    length = buffer[0] * 256 + buffer[1];
    if (buffer.length >= length + 2) {
        data = buffer.slice(2, length - 2);
        code = buffer.slice(length - 2);
        if (!(0, socket_1.checkCode)(config.key, data, code)) {
            receive.socket.end((0, socket_1.pack)({ code: 2 }));
            return;
        }
        try {
            const payload = JSON.parse(data.slice(6).toString());
            const rawResult = await receiveCommand(payload).catch((err) => {
                sentry_1.default.captureException(err, (scope) => {
                    scope.setTags({
                        phase: 'receiveCommand',
                    });
                    return scope;
                });
                if (payload.command) {
                    throw new Error(`Query error on '${payload.command}': ${err.message}`);
                }
                else {
                    throw new Error(`Query error: ${err.message}`);
                }
            });
            const result = parseResult(rawResult);
            logger_1.logger.debug('Result: ' + JSON.stringify(result, null, 2));
            receive.socket.end((0, socket_1.pack)({ code: 0, data: result }));
        }
        catch (err) {
            if (err instanceof Error) {
                logger_1.logger.error(err.message);
                receive.socket.end((0, socket_1.pack)({ code: err.message === 'Invalid command' ? 1 : -1 }));
            }
        }
    }
};
const server = (0, net_1.createServer)((socket) => {
    const receive = {
        data: Buffer.from(''),
        socket,
    };
    socket.on('data', (data) => {
        receive.data = Buffer.concat([receive.data, data]);
        checkData(receive).catch((err) => {
            sentry_1.default.captureException(err, (scope) => {
                scope.setTags({
                    phase: 'checkData',
                });
                return scope;
            });
            logger_1.logger.error('[checkData] ' + err.message);
        });
    });
    socket.on('error', (err) => {
        if ('code' in err) {
            switch (err.code) {
                case 'ECONNRESET':
                    logger_1.logger.debug('[socket:error] Socket error: ' + err.message);
                    break;
                default:
                    sentry_1.default.captureException(err, (scope) => {
                        scope.setTags({
                            phase: 'socket:error',
                        });
                        return scope;
                    });
                    logger_1.logger.error('[socket:error] Socket error: ' + err.message);
            }
        }
    });
});
server.on('error', (err) => {
    sentry_1.default.captureException(err, (scope) => {
        scope.setTags({
            phase: 'server:error',
        });
        return scope;
    });
    logger_1.logger.error('[server:error] TCP server error: ' + err.message);
    throw err;
});
const startServer = async () => {
    logger_1.logger.info(`Running ssmgr-trojan-client v${version_1.version}`);
    const database = (0, db_1.getDatabase)();
    await database.authenticate();
    await database.sync();
    config = (0, config_1.getConfig)();
    if (config.debug) {
        (0, logger_1.enableDebug)();
    }
    logger_1.logger.debug('%j', config);
    trojanClient = await (0, api_client_1.getClient)(config);
    if (config.fakeWebsite) {
        logger_1.logger.info('Initializing the fake website, listening on ' + config.fakeWebsite);
        fakeWebsiteProcess = (0, trojan_1.startFakeWebsite)(config.fakeWebsite);
        fakeWebsiteProcess.on('exit', (code) => {
            fakeWebsiteProcess = null;
            if (code && code > 0) {
                throw new Error(`Fake website process exited unexpectedly with code ${code}`);
            }
        });
        await new Promise((resolve) => setTimeout(resolve, 1000));
    }
    if (config.trojanConfig) {
        trojanProcess = (0, trojan_1.startTrojan)(config.apiHost, config.apiPort, config.trojanConfig);
        trojanProcess.on('error', (error) => {
            logger_1.trojanLogger.error(error.message);
            sentry_1.default.captureException(error, (scope) => {
                scope.setTags({
                    phase: 'trojan:error',
                });
                return scope;
            });
            if (trojanProcess) {
                trojanProcess.kill(1);
            }
        });
        trojanProcess.on('exit', (code) => {
            trojanProcess = null;
            if (code && code > 0) {
                throw new Error(`trojan-go process exited unexpectedly with code ${code}`);
            }
            else {
                throw new Error(`trojan-go process exited unexpectedly with code ${code || 'null'}`);
            }
        });
        trojanProcess.once('api-service-ready', () => {
            logger_1.logger.info('trojan-go API service is ready');
            trojanClient
                .init({
                onTickError: (err) => {
                    logger_1.logger.error('[trojanClient:onTickError] ' + err.message);
                    sentry_1.default.captureException(err, (scope) => {
                        scope.setTags({
                            phase: 'trojanClient:onTickError',
                        });
                        return scope;
                    });
                },
            })
                .catch((err) => {
                logger_1.logger.error('[trojanClient:init] ' + err.message);
                sentry_1.default.captureException(err, (scope) => {
                    scope.setTags({
                        phase: 'trojanClient:init',
                    });
                    return scope;
                });
                throw err;
            });
        });
    }
    else {
        await trojanClient.init({
            onTickError: (err) => {
                logger_1.logger.error('[trojanClient:onTickError] ' + err.message);
                sentry_1.default.captureException(err, (scope) => {
                    scope.setTags({
                        phase: 'trojanClient:onTickError',
                    });
                    return scope;
                });
            },
        });
    }
    server.listen(config.port, config.addr, () => {
        logger_1.logger.info(`Client is listening on ${config.addr}:${config.port}`);
    });
};
startServer().catch((e) => {
    if (e instanceof Error) {
        logger_1.logger.error('[startServer] ' + e.message);
        sentry_1.default.captureException(e, (scope) => {
            scope.setTags({
                phase: 'startServer',
            });
            return scope;
        });
    }
    else {
        logger_1.logger.error('[startServer] ' + e);
    }
    logger_1.logger.error('FATAL ERROR. TERMINATED.');
    process.exit(1);
});
(0, death_1.default)({ uncaughtException: true })((signal, err, origin) => {
    function report(error) {
        sentry_1.default.captureException(error, (scope) => {
            scope.setTags({
                phase: 'onDeath',
            });
            return scope;
        });
    }
    if (signal === 'uncaughtException') {
        if (err && origin) {
            report(err);
            logger_1.logger.error(err);
            logger_1.logger.error(`Received an uncaught exception. Terminating the service...`);
        }
    }
    else {
        logger_1.logger.info(`Received ${signal}. Terminating the service...`);
    }
    trojanClient.disconnect();
    if (trojanProcess) {
        trojanProcess.kill(0);
    }
    if (fakeWebsiteProcess) {
        fakeWebsiteProcess.kill(0);
    }
    server.close();
    (0, db_1.getDatabase)()
        .close()
        .finally(() => {
        process.exit(0);
    });
});
//# sourceMappingURL=app.js.map